from __future__ import annotations

import asyncio
import sys
from collections.abc import Mapping
from concurrent.futures import Executor, ThreadPoolExecutor
from typing import AsyncIterator, Optional, Sequence

from grpclib.client import Channel

from ._tar_stream import TarGzParser
from .api.afslet.v1.afslet_grpc import AfsletStub
from .api.afslet.v1 import afslet_pb2 as pb
from .api.afsproxy.v1.afsproxy_grpc import AfsProxyStub
from .api.afsproxy.v1 import afsproxy_pb2 as proxypb
from .models import (
    AcceptedEvent,
    DoneEvent,
    ExecuteEvent,
    ExecuteInput,
    ExtraDirectory,
    ExtraFile,
    ExtraSymlink,
    LogEvent,
    ResultEvent,
    RuntimeStatus,
    TarArchiveStart,
    ProxyStatusEvent,
    ProxyStatusSummary,
    ProxyLayerInfo,
    ProxyLayerstoreInstance,
    ProxyAfsletInstance,
    ProxyStatusError,
    ReconcileImageInput,
    ReconcileImageResult,
)


class AfsClient:
    """Asyncio + grpclib client for afs_proxy/afslet."""

    def __init__(
        self,
        host: str,
        port: int,
        *,
        ssl: Optional[bool] = None,
        tar_executor: Optional[Executor] = None,
    ) -> None:
        self._channel = Channel(host=host, port=port, ssl=ssl)
        self._stub = AfsletStub(self._channel)
        self._proxy_stub = AfsProxyStub(self._channel)
        self._owned_executor: Optional[ThreadPoolExecutor] = None
        if tar_executor is None:
            self._owned_executor = ThreadPoolExecutor(max_workers=1, thread_name_prefix="afs-sdk-tar")
            self._tar_executor: Executor = self._owned_executor
        else:
            self._tar_executor = tar_executor

    async def close(self) -> None:
        self._channel.close()
        if self._owned_executor is not None:
            self._owned_executor.shutdown(wait=True, cancel_futures=False)
            self._owned_executor = None

    async def __aenter__(self) -> "AfsClient":
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:
        await self.close()

    async def get_runtime_status(self) -> RuntimeStatus:
        resp = await self._stub.GetRuntimeStatus(pb.GetRuntimeStatusRequest())
        return RuntimeStatus(
            running_containers=resp.running_containers,
            limit_cpu_cores=resp.limit_cpu_cores,
            limit_memory_mb=resp.limit_memory_mb,
            used_cpu_cores=resp.used_cpu_cores,
            used_memory_mb=resp.used_memory_mb,
            available_cpu_cores=resp.available_cpu_cores,
            available_memory_mb=resp.available_memory_mb,
        )

    async def iter_runtime_status(self) -> AsyncIterator[RuntimeStatus]:
        yield await self.get_runtime_status()

    async def status(
        self,
        *,
        include_layerstores: bool = True,
        include_afslets: bool = True,
    ) -> AsyncIterator[ProxyStatusEvent]:
        req = proxypb.StatusRequest(
            include_layerstores=include_layerstores,
            include_afslets=include_afslets,
        )
        call = self._proxy_stub.Status(req)
        if hasattr(call, "__aiter__"):
            async_iter = call
            sync_iter = None
        else:
            result = await call
            if hasattr(result, "__aiter__"):
                async_iter = result
                sync_iter = None
            else:
                async_iter = None
                sync_iter = result

        if async_iter is not None:
            async for resp in async_iter:
                kind = resp.WhichOneof("payload")
                if kind == "summary":
                    p = resp.summary
                    yield ProxyStatusSummary(
                        layerstore_instances=p.layerstore_instances,
                        afslet_instances=p.afslet_instances,
                        afslet_reachable=p.afslet_reachable,
                        total_layers=p.total_layers,
                    )
                elif kind == "layerstore":
                    p = resp.layerstore
                    layers = [ProxyLayerInfo(digest=x.digest, afs_size=x.afs_size) for x in p.layers]
                    yield ProxyLayerstoreInstance(
                        node_id=p.node_id,
                        endpoint=p.endpoint,
                        last_seen_unix=p.last_seen_unix,
                        cache_max_bytes=p.cache_max_bytes,
                        layers=layers,
                        cached_images=list(p.cached_images),
                    )
                elif kind == "afslet":
                    p = resp.afslet
                    yield ProxyAfsletInstance(
                        endpoint=p.endpoint,
                        reachable=p.reachable,
                        error=p.error,
                        running_containers=p.running_containers,
                        limit_cpu_cores=p.limit_cpu_cores,
                        limit_memory_mb=p.limit_memory_mb,
                        used_cpu_cores=p.used_cpu_cores,
                        used_memory_mb=p.used_memory_mb,
                        available_cpu_cores=p.available_cpu_cores,
                        available_memory_mb=p.available_memory_mb,
                    )
                elif kind == "error":
                    p = resp.error
                    yield ProxyStatusError(source=p.source, message=p.message)
            return

        for resp in sync_iter:
            kind = resp.WhichOneof("payload")
            if kind == "summary":
                p = resp.summary
                yield ProxyStatusSummary(
                    layerstore_instances=p.layerstore_instances,
                    afslet_instances=p.afslet_instances,
                    afslet_reachable=p.afslet_reachable,
                    total_layers=p.total_layers,
                )
            elif kind == "layerstore":
                p = resp.layerstore
                layers = [ProxyLayerInfo(digest=x.digest, afs_size=x.afs_size) for x in p.layers]
                yield ProxyLayerstoreInstance(
                    node_id=p.node_id,
                    endpoint=p.endpoint,
                    last_seen_unix=p.last_seen_unix,
                    cache_max_bytes=p.cache_max_bytes,
                    layers=layers,
                    cached_images=list(p.cached_images),
                )
            elif kind == "afslet":
                p = resp.afslet
                yield ProxyAfsletInstance(
                    endpoint=p.endpoint,
                    reachable=p.reachable,
                    error=p.error,
                    running_containers=p.running_containers,
                    limit_cpu_cores=p.limit_cpu_cores,
                    limit_memory_mb=p.limit_memory_mb,
                    used_cpu_cores=p.used_cpu_cores,
                    used_memory_mb=p.used_memory_mb,
                    available_cpu_cores=p.available_cpu_cores,
                    available_memory_mb=p.available_memory_mb,
                )
            elif kind == "error":
                p = resp.error
                yield ProxyStatusError(source=p.source, message=p.message)

    async def reconcile_image(self, request: ReconcileImageInput) -> ReconcileImageResult:
        if request.replica < 0:
            raise ValueError("replica must be >= 0")
        if not request.image.strip():
            raise ValueError("image is required")
        resp = await self._proxy_stub.ReconcileImageReplica(
            proxypb.ReconcileImageReplicaRequest(
                image=request.image,
                tag=request.tag,
                platform_os=request.platform_os,
                platform_arch=request.platform_arch,
                platform_variant=request.platform_variant,
                replica=request.replica,
            )
        )
        return ReconcileImageResult(
            image_key=resp.image_key,
            current_replica=resp.current_replica,
            requested_replica=resp.requested_replica,
            ensured=resp.ensured,
        )

    async def raw_execute(self, requests: AsyncIterator[pb.ExecuteRequest]) -> AsyncIterator[pb.ExecuteResponse]:
        """Low-level Execute API using raw protobuf frames."""
        async with self._stub.Execute.open() as stream:
            await stream.send_request()

            async def _sender() -> None:
                async for req in requests:
                    await stream.send_message(req)
                await stream.end()

            sender_task = asyncio.create_task(_sender())
            try:
                while True:
                    msg = await stream.recv_message()
                    if msg is None:
                        break
                    yield msg
            finally:
                active_exc = sys.exc_info()[1]
                if not sender_task.done():
                    sender_task.cancel()
                try:
                    await sender_task
                except asyncio.CancelledError:
                    pass
                except BaseException:
                    if active_exc is None:
                        raise

    async def execute(self, request: ExecuteInput) -> AsyncIterator[ExecuteEvent]:
        """High-level Execute API with typed request/event models."""
        req_stream = self._iter_raw_requests(request)
        parser: Optional[TarGzParser] = None
        async for resp in self.raw_execute(req_stream):
            kind = resp.WhichOneof("payload")
            if kind == "accepted":
                yield AcceptedEvent(accepted=resp.accepted.accepted)
                continue
            if kind == "log":
                yield LogEvent(source=resp.log.source, message=resp.log.message)
                continue
            if kind == "result":
                yield ResultEvent(success=resp.result.success, exit_code=resp.result.exit_code, error=resp.result.error)
                continue
            if kind == "tar_header":
                yield TarArchiveStart(name=resp.tar_header.name)
                if parser is None:
                    parser = TarGzParser(asyncio.get_running_loop(), self._tar_executor)
                    parser.start()
                continue
            if kind == "tar_chunk":
                if parser is None:
                    parser = TarGzParser(asyncio.get_running_loop(), self._tar_executor)
                    parser.start()
                await parser.feed(resp.tar_chunk.data)
                for event in await parser.drain_available():
                    yield event
                continue
            if kind == "done":
                if parser is not None:
                    await parser.finish_input()
                    async for event in parser.iter_until_end():
                        yield event
                    await parser.wait_closed()
                yield DoneEvent()
                return

    async def _iter_raw_requests(self, req: ExecuteInput) -> AsyncIterator[pb.ExecuteRequest]:
        if req.cpu_cores <= 0:
            raise ValueError("cpu_cores must be > 0")
        if req.memory_mb <= 0:
            raise ValueError("memory_mb must be > 0")
        if req.timeout_ms <= 0:
            raise ValueError("timeout_ms must be > 0")
        if req.upload_chunk_size <= 0:
            raise ValueError("upload_chunk_size must be > 0")
        if not req.image.strip():
            raise ValueError("image is required")
        env = _normalize_env_entries(req.env)

        yield pb.ExecuteRequest(
            start=pb.StartRequest(
                image=req.image,
                tag=req.tag,
                command=list(req.command),
                env=env,
                cpu_cores=req.cpu_cores,
                memory_mb=req.memory_mb,
                timeout_ms=req.timeout_ms,
                discovery_addr=req.discovery_addr,
                force_local_fetch=req.force_local_fetch,
                node_id=req.node_id,
                platform_os=req.platform_os,
                platform_arch=req.platform_arch,
                platform_variant=req.platform_variant,
                proxy_dispatch_max_retries=req.proxy_dispatch_max_retries,
                proxy_dispatch_backoff_ms=req.proxy_dispatch_backoff_ms,
                fuse_max_read_ahead_bytes=req.fuse_max_read_ahead_bytes,
            )
        )

        for entry in req.extra_entries:
            if isinstance(entry, ExtraDirectory):
                yield pb.ExecuteRequest(
                    file_begin=pb.FileEntryBegin(
                        path=entry.path,
                        type=pb.FILE_TYPE_DIR,
                        mode=entry.mode,
                        uid=entry.uid,
                        gid=entry.gid,
                        mtime_unix=entry.mtime_unix,
                        symlink_target="",
                    )
                )
                yield pb.ExecuteRequest(file_end=pb.FileEntryEnd())
                continue

            if isinstance(entry, ExtraSymlink):
                yield pb.ExecuteRequest(
                    file_begin=pb.FileEntryBegin(
                        path=entry.path,
                        type=pb.FILE_TYPE_SYMLINK,
                        mode=entry.mode,
                        uid=entry.uid,
                        gid=entry.gid,
                        mtime_unix=entry.mtime_unix,
                        symlink_target=entry.target,
                    )
                )
                yield pb.ExecuteRequest(file_end=pb.FileEntryEnd())
                continue

            if isinstance(entry, ExtraFile):
                yield pb.ExecuteRequest(
                    file_begin=pb.FileEntryBegin(
                        path=entry.path,
                        type=pb.FILE_TYPE_FILE,
                        mode=entry.mode,
                        uid=entry.uid,
                        gid=entry.gid,
                        mtime_unix=entry.mtime_unix,
                        symlink_target="",
                    )
                )
                if isinstance(entry.content, (bytes, bytearray)):
                    data = bytes(entry.content)
                    step = req.upload_chunk_size
                    for i in range(0, len(data), step):
                        yield pb.ExecuteRequest(file_chunk=pb.FileChunk(data=data[i : i + step]))
                else:
                    async for chunk in entry.content:
                        if not isinstance(chunk, (bytes, bytearray)):
                            raise TypeError(f"file content chunk must be bytes, got {type(chunk)!r}")
                        if chunk:
                            yield pb.ExecuteRequest(file_chunk=pb.FileChunk(data=bytes(chunk)))
                yield pb.ExecuteRequest(file_end=pb.FileEntryEnd())
                continue

            raise TypeError(f"unsupported extra entry type: {type(entry)!r}")


def _normalize_env_entries(env: Mapping[str, str] | Sequence[str]) -> list[str]:
    if isinstance(env, Mapping):
        return [f"{key}={value}" for key, value in env.items()]
    if isinstance(env, str):
        raise TypeError("env must be a mapping or a sequence of KEY=VALUE strings")

    out: list[str] = []
    for entry in env:
        if not isinstance(entry, str):
            raise TypeError(f"env entry must be str, got {type(entry)!r}")
        out.append(entry)
    return out



from __future__ import annotations

import argparse
import asyncio

from afs_sdk import AfsClient, ReconcileImageInput


async def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("--addr", default="127.0.0.1:62051", help="afs_proxy address")
    parser.add_argument("--image", required=True, help="image name")
    parser.add_argument("--tag", default="", help="image tag")
    parser.add_argument("--platform-os", default="linux")
    parser.add_argument("--platform-arch", default="amd64")
    parser.add_argument("--platform-variant", default="")
    parser.add_argument("--replica", type=int, required=True, help="target replica, must be >= 0")
    args = parser.parse_args()

    host, port_str = args.addr.rsplit(":", 1)
    port = int(port_str)
    req = ReconcileImageInput(
        image=args.image,
        tag=args.tag,
        platform_os=args.platform_os,
        platform_arch=args.platform_arch,
        platform_variant=args.platform_variant,
        replica=args.replica,
    )

    async with AfsClient(host, port) as client:
        resp = await client.reconcile_image(req)
        print(
            f"image_key={resp.image_key} current_replica={resp.current_replica} "
            f"requested_replica={resp.requested_replica} ensured={resp.ensured}"
        )


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations

from dataclasses import dataclass
from functools import lru_cache
from importlib.metadata import PackageNotFoundError, version as package_version
from pathlib import Path
import re
import subprocess

_DESCRIBE_PATTERN = re.compile(
    r"^(?P<tag>.+)-(?P<distance>\d+)-g(?P<commit>[0-9a-f]+)(?P<dirty>-dirty)?$"
)
_COMMIT_PATTERN = re.compile(r"^(?P<commit>[0-9a-f]{7,})(?P<dirty>-dirty)?$")


@dataclass(frozen=True, slots=True)
class VersionInfo:
    version: str
    tag: str | None
    commit_sha: str | None
    distance: int | None
    dirty: bool


def _repo_root() -> Path:
    return Path(__file__).resolve().parents[2]


def _describe_from_git() -> str | None:
    try:
        completed = subprocess.run(
            [
                "git",
                "describe",
                "--tags",
                "--long",
                "--always",
                "--dirty",
                "--match",
                "v*",
            ],
            cwd=_repo_root(),
            check=True,
            capture_output=True,
            text=True,
        )
    except (FileNotFoundError, OSError, subprocess.CalledProcessError):
        return None

    describe = completed.stdout.strip()
    return describe or None


def _fallback_version() -> str:
    try:
        return package_version("afs-sdk")
    except PackageNotFoundError:
        return "0.0.0+unknown"


def _parse_version(describe: str) -> VersionInfo:
    describe_match = _DESCRIBE_PATTERN.fullmatch(describe)
    if describe_match is not None:
        return VersionInfo(
            version=describe,
            tag=describe_match.group("tag"),
            commit_sha=describe_match.group("commit"),
            distance=int(describe_match.group("distance")),
            dirty=describe_match.group("dirty") is not None,
        )

    commit_match = _COMMIT_PATTERN.fullmatch(describe)
    if commit_match is not None:
        return VersionInfo(
            version=describe,
            tag=None,
            commit_sha=commit_match.group("commit"),
            distance=None,
            dirty=commit_match.group("dirty") is not None,
        )

    return VersionInfo(
        version=describe,
        tag=describe,
        commit_sha=None,
        distance=None,
        dirty=describe.endswith("-dirty"),
    )


@lru_cache(maxsize=1)
def get_version_info() -> VersionInfo:
    describe = _describe_from_git()
    if describe is not None:
        return _parse_version(describe)

    version = _fallback_version()
    return VersionInfo(
        version=version,
        tag=version,
        commit_sha=None,
        distance=None,
        dirty=False,
    )


def get_version() -> str:
    return get_version_info().version


__version__ = get_version()




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/afsproxy/v1/afsproxy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x61pi/afsproxy/v1/afsproxy.proto\x12\x0b\x61\x66sproxy.v1\"E\n\rStatusRequest\x12\x1b\n\x13include_layerstores\x18\x01 \x01(\x08\x12\x17\n\x0finclude_afslets\x18\x02 \x01(\x08\"\x91\x01\n\x1cReconcileImageReplicaRequest\x12\r\n\x05image\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x13\n\x0bplatform_os\x18\x03 \x01(\t\x12\x15\n\rplatform_arch\x18\x04 \x01(\t\x12\x18\n\x10platform_variant\x18\x05 \x01(\t\x12\x0f\n\x07replica\x18\x06 \x01(\x05\"w\n\x1dReconcileImageReplicaResponse\x12\x11\n\timage_key\x18\x01 \x01(\t\x12\x17\n\x0f\x63urrent_replica\x18\x02 \x01(\x05\x12\x19\n\x11requested_replica\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nsured\x18\x04 \x01(\x08\"w\n\rStatusSummary\x12\x1c\n\x14layerstore_instances\x18\x01 \x01(\x03\x12\x18\n\x10\x61\x66slet_instances\x18\x02 \x01(\x03\x12\x18\n\x10\x61\x66slet_reachable\x18\x03 \x01(\x03\x12\x14\n\x0ctotal_layers\x18\x04 \x01(\x03\"\xa7\x01\n\x12LayerstoreInstance\x12\x0f\n\x07node_id\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12\x16\n\x0elast_seen_unix\x18\x03 \x01(\x03\x12\x17\n\x0f\x63\x61\x63he_max_bytes\x18\x04 \x01(\x03\x12&\n\x06layers\x18\x05 \x03(\x0b\x32\x16.afsproxy.v1.LayerInfo\x12\x15\n\rcached_images\x18\x06 \x03(\t\"-\n\tLayerInfo\x12\x0e\n\x06\x64igest\x18\x01 \x01(\t\x12\x10\n\x08\x61\x66s_size\x18\x02 \x01(\x03\"\xfc\x01\n\x0e\x41\x66sletInstance\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\t\x12\x11\n\treachable\x18\x02 \x01(\x08\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12\x1a\n\x12running_containers\x18\x04 \x01(\x03\x12\x17\n\x0flimit_cpu_cores\x18\x05 \x01(\x03\x12\x17\n\x0flimit_memory_mb\x18\x06 \x01(\x03\x12\x16\n\x0eused_cpu_cores\x18\x07 \x01(\x03\x12\x16\n\x0eused_memory_mb\x18\x08 \x01(\x03\x12\x1b\n\x13\x61vailable_cpu_cores\x18\t \x01(\x03\x12\x1b\n\x13\x61vailable_memory_mb\x18\n \x01(\x03\".\n\x0bStatusError\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xdb\x01\n\x0eStatusResponse\x12-\n\x07summary\x18\x01 \x01(\x0b\x32\x1a.afsproxy.v1.StatusSummaryH\x00\x12\x35\n\nlayerstore\x18\x02 \x01(\x0b\x32\x1f.afsproxy.v1.LayerstoreInstanceH\x00\x12-\n\x06\x61\x66slet\x18\x03 \x01(\x0b\x32\x1b.afsproxy.v1.AfsletInstanceH\x00\x12)\n\x05\x65rror\x18\x04 \x01(\x0b\x32\x18.afsproxy.v1.StatusErrorH\x00\x42\t\n\x07payload2\xbf\x01\n\x08\x41\x66sProxy\x12\x43\n\x06Status\x12\x1a.afsproxy.v1.StatusRequest\x1a\x1b.afsproxy.v1.StatusResponse0\x01\x12n\n\x15ReconcileImageReplica\x12).afsproxy.v1.ReconcileImageReplicaRequest\x1a*.afsproxy.v1.ReconcileImageReplicaResponseB2Z0github.com/reyoung/afs/pkg/afsproxypb;afsproxypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api.afsproxy.v1.afsproxy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/reyoung/afs/pkg/afsproxypb;afsproxypb'
  _globals['_STATUSREQUEST']._serialized_start=47
  _globals['_STATUSREQUEST']._serialized_end=116
  _globals['_RECONCILEIMAGEREPLICAREQUEST']._serialized_start=119
  _globals['_RECONCILEIMAGEREPLICAREQUEST']._serialized_end=264
  _globals['_RECONCILEIMAGEREPLICARESPONSE']._serialized_start=266
  _globals['_RECONCILEIMAGEREPLICARESPONSE']._serialized_end=385
  _globals['_STATUSSUMMARY']._serialized_start=387
  _globals['_STATUSSUMMARY']._serialized_end=506
  _globals['_LAYERSTOREINSTANCE']._serialized_start=509
  _globals['_LAYERSTOREINSTANCE']._serialized_end=676
  _globals['_LAYERINFO']._serialized_start=678
  _globals['_LAYERINFO']._serialized_end=723
  _globals['_AFSLETINSTANCE']._serialized_start=726
  _globals['_AFSLETINSTANCE']._serialized_end=978
  _globals['_STATUSERROR']._serialized_start=980
  _globals['_STATUSERROR']._serialized_end=1026
  _globals['_STATUSRESPONSE']._serialized_start=1029
  _globals['_STATUSRESPONSE']._serialized_end=1248
  _globals['_AFSPROXY']._serialized_start=1251
  _globals['_AFSPROXY']._serialized_end=1442
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/afslet/v1/afslet.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61pi/afslet/v1/afslet.proto\x12\x08\x61\x66sletpb\"\xcb\x01\n\x0e\x45xecuteRequest\x12\'\n\x05start\x18\x01 \x01(\x0b\x32\x16.afsletpb.StartRequestH\x00\x12.\n\nfile_begin\x18\x02 \x01(\x0b\x32\x18.afsletpb.FileEntryBeginH\x00\x12)\n\nfile_chunk\x18\x03 \x01(\x0b\x32\x13.afsletpb.FileChunkH\x00\x12*\n\x08\x66ile_end\x18\x04 \x01(\x0b\x32\x16.afsletpb.FileEntryEndH\x00\x42\t\n\x07payload\"\xf6\x02\n\x0cStartRequest\x12\r\n\x05image\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x03 \x03(\t\x12\x11\n\tcpu_cores\x18\x04 \x01(\x03\x12\x11\n\tmemory_mb\x18\x05 \x01(\x03\x12\x12\n\ntimeout_ms\x18\x06 \x01(\x03\x12\x16\n\x0e\x64iscovery_addr\x18\x07 \x01(\t\x12\x19\n\x11\x66orce_local_fetch\x18\x08 \x01(\x08\x12\x0f\n\x07node_id\x18\t \x01(\t\x12\x13\n\x0bplatform_os\x18\n \x01(\t\x12\x15\n\rplatform_arch\x18\x0b \x01(\t\x12\x18\n\x10platform_variant\x18\x0c \x01(\t\x12\"\n\x1aproxy_dispatch_max_retries\x18\r \x01(\x03\x12!\n\x19proxy_dispatch_backoff_ms\x18\x0e \x01(\x03\x12\x0b\n\x03\x65nv\x18\x0f \x03(\t\x12!\n\x19\x66use_max_read_ahead_bytes\x18\x10 \x01(\x03\"\x94\x01\n\x0e\x46ileEntryBegin\x12\x0c\n\x04path\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0e\x32\x12.afsletpb.FileType\x12\x0c\n\x04mode\x18\x03 \x01(\r\x12\x0b\n\x03uid\x18\x04 \x01(\r\x12\x0b\n\x03gid\x18\x05 \x01(\r\x12\x12\n\nmtime_unix\x18\x06 \x01(\x03\x12\x16\n\x0esymlink_target\x18\x07 \x01(\t\"\x19\n\tFileChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x0e\n\x0c\x46ileEntryEnd\"\x81\x02\n\x0f\x45xecuteResponse\x12#\n\x03log\x18\x01 \x01(\x0b\x32\x14.afsletpb.LogMessageH\x00\x12\"\n\x06result\x18\x02 \x01(\x0b\x32\x10.afsletpb.ResultH\x00\x12)\n\ntar_header\x18\x03 \x01(\x0b\x32\x13.afsletpb.TarHeaderH\x00\x12\'\n\ttar_chunk\x18\x04 \x01(\x0b\x32\x12.afsletpb.TarChunkH\x00\x12\x1e\n\x04\x64one\x18\x05 \x01(\x0b\x32\x0e.afsletpb.DoneH\x00\x12&\n\x08\x61\x63\x63\x65pted\x18\x06 \x01(\x0b\x32\x12.afsletpb.AcceptedH\x00\x42\t\n\x07payload\"\x1c\n\x08\x41\x63\x63\x65pted\x12\x10\n\x08\x61\x63\x63\x65pted\x18\x01 \x01(\x08\"-\n\nLogMessage\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\";\n\x06Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\texit_code\x18\x02 \x01(\x05\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"\x19\n\tTarHeader\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x18\n\x08TarChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x06\n\x04\x44one\"\x19\n\x17GetRuntimeStatusRequest\"\xd2\x01\n\x18GetRuntimeStatusResponse\x12\x1a\n\x12running_containers\x18\x01 \x01(\x03\x12\x17\n\x0flimit_cpu_cores\x18\x02 \x01(\x03\x12\x17\n\x0flimit_memory_mb\x18\x03 \x01(\x03\x12\x16\n\x0eused_cpu_cores\x18\x04 \x01(\x03\x12\x16\n\x0eused_memory_mb\x18\x05 \x01(\x03\x12\x1b\n\x13\x61vailable_cpu_cores\x18\x06 \x01(\x03\x12\x1b\n\x13\x61vailable_memory_mb\x18\x07 \x01(\x03*H\n\x08\x46ileType\x12\x11\n\rFILE_TYPE_DIR\x10\x00\x12\x12\n\x0e\x46ILE_TYPE_FILE\x10\x01\x12\x15\n\x11\x46ILE_TYPE_SYMLINK\x10\x02\x32\xa7\x01\n\x06\x41\x66slet\x12\x42\n\x07\x45xecute\x12\x18.afsletpb.ExecuteRequest\x1a\x19.afsletpb.ExecuteResponse(\x01\x30\x01\x12Y\n\x10GetRuntimeStatus\x12!.afsletpb.GetRuntimeStatusRequest\x1a\".afsletpb.GetRuntimeStatusResponseB.Z,github.com/reyoung/afs/pkg/afsletpb;afsletpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'api.afslet.v1.afslet_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/reyoung/afs/pkg/afsletpb;afsletpb'
  _globals['_FILETYPE']._serialized_start=1516
  _globals['_FILETYPE']._serialized_end=1588
  _globals['_EXECUTEREQUEST']._serialized_start=41
  _globals['_EXECUTEREQUEST']._serialized_end=244
  _globals['_STARTREQUEST']._serialized_start=247
  _globals['_STARTREQUEST']._serialized_end=621
  _globals['_FILEENTRYBEGIN']._serialized_start=624
  _globals['_FILEENTRYBEGIN']._serialized_end=772
  _globals['_FILECHUNK']._serialized_start=774
  _globals['_FILECHUNK']._serialized_end=799
  _globals['_FILEENTRYEND']._serialized_start=801
  _globals['_FILEENTRYEND']._serialized_end=815
  _globals['_EXECUTERESPONSE']._serialized_start=818
  _globals['_EXECUTERESPONSE']._serialized_end=1075
  _globals['_ACCEPTED']._serialized_start=1077
  _globals['_ACCEPTED']._serialized_end=1105
  _globals['_LOGMESSAGE']._serialized_start=1107
  _globals['_LOGMESSAGE']._serialized_end=1152
  _globals['_RESULT']._serialized_start=1154
  _globals['_RESULT']._serialized_end=1213
  _globals['_TARHEADER']._serialized_start=1215
  _globals['_TARHEADER']._serialized_end=1240
  _globals['_TARCHUNK']._serialized_start=1242
  _globals['_TARCHUNK']._serialized_end=1266
  _globals['_DONE']._serialized_start=1268
  _globals['_DONE']._serialized_end=1274
  _globals['_GETRUNTIMESTATUSREQUEST']._serialized_start=1276
  _globals['_GETRUNTIMESTATUSREQUEST']._serialized_end=1301
  _globals['_GETRUNTIMESTATUSRESPONSE']._serialized_start=1304
  _globals['_GETRUNTIMESTATUSRESPONSE']._serialized_end=1514
  _globals['_AFSLET']._serialized_start=1591
  _globals['_AFSLET']._serialized_end=1758
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import asyncio
import gzip
import io
import queue
import tarfile
from concurrent.futures import Executor
from multiprocessing import Manager
from multiprocessing.managers import SyncManager
from typing import Any, AsyncIterator, Optional

from .models import TarDirectory, TarFilePart, TarSymlink


_SENTINEL = None


def _tar_worker(in_q: Any, out_q: Any, part_size: int) -> None:
    class ChunkReader(io.RawIOBase):
        def __init__(self, q: Any) -> None:
            self._q = q
            self._buf = b""
            self._eof = False

        def readable(self) -> bool:
            return True

        def readinto(self, b: bytearray) -> int:
            if self._eof and not self._buf:
                return 0
            view = memoryview(b)
            written = 0
            while written < len(view):
                if self._buf:
                    n = min(len(self._buf), len(view) - written)
                    view[written : written + n] = self._buf[:n]
                    self._buf = self._buf[n:]
                    written += n
                    if written > 0:
                        return written
                if self._eof:
                    return written
                chunk = self._q.get()
                if chunk is _SENTINEL:
                    self._eof = True
                elif isinstance(chunk, (bytes, bytearray)):
                    self._buf = bytes(chunk)
                else:
                    raise TypeError(f"unexpected chunk type: {type(chunk)!r}")
            return written

    def put_event(kind: str, payload: dict[str, Any]) -> None:
        out_q.put((kind, payload))

    try:
        raw = ChunkReader(in_q)
        buffered = io.BufferedReader(raw, buffer_size=part_size)
        gz = gzip.GzipFile(fileobj=buffered, mode="rb")
        tf = tarfile.open(fileobj=gz, mode="r|")
        for member in tf:
            base = {
                "path": member.name,
                "mode": int(member.mode),
                "uid": int(member.uid),
                "gid": int(member.gid),
                "uname": member.uname or "",
                "gname": member.gname or "",
                "mtime_unix": int(member.mtime),
            }
            if member.isdir():
                put_event("tar_dir", base)
                continue
            if member.issym():
                payload = dict(base)
                payload["target"] = member.linkname or ""
                put_event("tar_symlink", payload)
                continue
            if member.isreg():
                f = tf.extractfile(member)
                if f is None:
                    payload = dict(base)
                    payload.update({"size": int(member.size), "offset": 0, "data": b"", "eof": True})
                    put_event("tar_file_part", payload)
                    continue
                offset = 0
                size = int(member.size)
                while True:
                    data = f.read(part_size)
                    if not data:
                        break
                    payload = dict(base)
                    payload.update(
                        {
                            "size": size,
                            "offset": offset,
                            "data": data,
                            "eof": False,
                        }
                    )
                    put_event("tar_file_part", payload)
                    offset += len(data)
                payload = dict(base)
                payload.update({"size": size, "offset": offset, "data": b"", "eof": True})
                put_event("tar_file_part", payload)
        out_q.put(("eof", {}))
    except Exception as err:
        out_q.put(("error", {"message": str(err)}))


class TarGzParser:
    def __init__(self, loop: asyncio.AbstractEventLoop, executor: Executor, part_size: int = 256 * 1024) -> None:
        self._loop = loop
        self._executor = executor
        self._part_size = part_size
        self._events: asyncio.Queue[object] = asyncio.Queue()
        self._poller_task: Optional[asyncio.Task[None]] = None
        self._worker_fut: Optional[asyncio.Future[None]] = None
        self._manager: Optional[SyncManager] = None

        if executor.__class__.__name__ == "ProcessPoolExecutor":
            self._manager = Manager()
            self._in_q = self._manager.Queue()
            self._out_q = self._manager.Queue()
        else:
            self._in_q = queue.Queue()
            self._out_q = queue.Queue()

    def start(self) -> None:
        if self._worker_fut is not None:
            return
        self._worker_fut = self._loop.run_in_executor(self._executor, _tar_worker, self._in_q, self._out_q, self._part_size)
        self._poller_task = self._loop.create_task(self._poll_out_queue())

    async def feed(self, chunk: bytes) -> None:
        await asyncio.to_thread(self._in_q.put, chunk)

    async def finish_input(self) -> None:
        await asyncio.to_thread(self._in_q.put, _SENTINEL)

    async def drain_available(self) -> list[object]:
        out: list[object] = []
        while True:
            try:
                out.append(self._events.get_nowait())
            except asyncio.QueueEmpty:
                return out

    async def iter_until_end(self) -> AsyncIterator[object]:
        while True:
            item = await self._events.get()
            if item is _SENTINEL:
                return
            if isinstance(item, Exception):
                raise item
            yield item

    async def wait_closed(self) -> None:
        if self._poller_task is not None:
            await self._poller_task
        if self._worker_fut is not None:
            await self._worker_fut
        if self._manager is not None:
            self._manager.shutdown()

    async def _poll_out_queue(self) -> None:
        while True:
            kind, payload = await asyncio.to_thread(self._out_q.get)
            if kind == "eof":
                await self._events.put(_SENTINEL)
                return
            if kind == "error":
                await self._events.put(RuntimeError(payload.get("message", "tar parse failed")))
                await self._events.put(_SENTINEL)
                return
            if kind == "tar_dir":
                await self._events.put(TarDirectory(**payload))
                continue
            if kind == "tar_symlink":
                await self._events.put(TarSymlink(**payload))
                continue
            if kind == "tar_file_part":
                await self._events.put(TarFilePart(**payload))
                continue
            await self._events.put(RuntimeError(f"unknown parser event kind: {kind}"))
            await self._events.put(_SENTINEL)
            return



from ._version import VersionInfo, __version__, get_version, get_version_info
from .client import AfsClient
from .models import (
    AcceptedEvent,
    DoneEvent,
    ExecuteEvent,
    ExecuteInput,
    ExtraDirectory,
    ExtraEntry,
    ExtraFile,
    ExtraSymlink,
    LogEvent,
    ProxyAfsletInstance,
    ProxyLayerInfo,
    ProxyLayerstoreInstance,
    ProxyStatusError,
    ProxyStatusEvent,
    ProxyStatusSummary,
    ReconcileImageInput,
    ReconcileImageResult,
    ResultEvent,
    RuntimeStatus,
    TarArchiveStart,
    TarDirectory,
    TarFilePart,
    TarSymlink,
)

__all__ = [
    "__version__",
    "AfsClient",
    "ExecuteInput",
    "ExtraEntry",
    "ExtraDirectory",
    "ExtraFile",
    "ExtraSymlink",
    "RuntimeStatus",
    "ProxyStatusEvent",
    "ProxyStatusSummary",
    "ProxyLayerInfo",
    "ProxyLayerstoreInstance",
    "ProxyAfsletInstance",
    "ProxyStatusError",
    "ReconcileImageInput",
    "ReconcileImageResult",
    "ExecuteEvent",
    "AcceptedEvent",
    "LogEvent",
    "ResultEvent",
    "TarArchiveStart",
    "TarDirectory",
    "TarSymlink",
    "TarFilePart",
    "DoneEvent",
    "VersionInfo",
    "get_version",
    "get_version_info",
]



from __future__ import annotations

from dataclasses import dataclass, field
from typing import AsyncIterator, Mapping, Sequence, Union


ByteChunks = AsyncIterator[bytes]


@dataclass(slots=True)
class ExtraDirectory:
    path: str
    mode: int = 0o755
    uid: int = 0
    gid: int = 0
    mtime_unix: int = 0


@dataclass(slots=True)
class ExtraSymlink:
    path: str
    target: str
    mode: int = 0o777
    uid: int = 0
    gid: int = 0
    mtime_unix: int = 0


@dataclass(slots=True)
class ExtraFile:
    path: str
    content: Union[bytes, ByteChunks]
    mode: int = 0o644
    uid: int = 0
    gid: int = 0
    mtime_unix: int = 0


ExtraEntry = Union[ExtraDirectory, ExtraSymlink, ExtraFile]


@dataclass(slots=True)
class ExecuteInput:
    image: str
    command: Sequence[str] = field(default_factory=tuple)
    env: Mapping[str, str] | Sequence[str] = field(default_factory=tuple)
    fuse_max_read_ahead_bytes: int = 0
    tag: str = ""
    cpu_cores: int = 1
    memory_mb: int = 256
    timeout_ms: int = 1000
    discovery_addr: str = ""
    force_local_fetch: bool = False
    node_id: str = ""
    platform_os: str = "linux"
    platform_arch: str = "amd64"
    platform_variant: str = ""
    proxy_dispatch_max_retries: int = 0
    proxy_dispatch_backoff_ms: int = 50
    extra_entries: Sequence[ExtraEntry] = field(default_factory=tuple)
    upload_chunk_size: int = 256 * 1024


@dataclass(slots=True)
class RuntimeStatus:
    running_containers: int
    limit_cpu_cores: int
    limit_memory_mb: int
    used_cpu_cores: int
    used_memory_mb: int
    available_cpu_cores: int
    available_memory_mb: int


@dataclass(slots=True)
class ProxyStatusSummary:
    layerstore_instances: int
    afslet_instances: int
    afslet_reachable: int
    total_layers: int


@dataclass(slots=True)
class ProxyLayerInfo:
    digest: str
    afs_size: int


@dataclass(slots=True)
class ProxyLayerstoreInstance:
    node_id: str
    endpoint: str
    last_seen_unix: int
    cache_max_bytes: int
    layers: list[ProxyLayerInfo]
    cached_images: list[str]


@dataclass(slots=True)
class ProxyAfsletInstance:
    endpoint: str
    reachable: bool
    error: str
    running_containers: int
    limit_cpu_cores: int
    limit_memory_mb: int
    used_cpu_cores: int
    used_memory_mb: int
    available_cpu_cores: int
    available_memory_mb: int


@dataclass(slots=True)
class ProxyStatusError:
    source: str
    message: str


@dataclass(slots=True)
class ReconcileImageInput:
    image: str
    replica: int
    tag: str = ""
    platform_os: str = "linux"
    platform_arch: str = "amd64"
    platform_variant: str = ""


@dataclass(slots=True)
class ReconcileImageResult:
    image_key: str
    current_replica: int
    requested_replica: int
    ensured: bool


@dataclass(slots=True)
class AcceptedEvent:
    accepted: bool


@dataclass(slots=True)
class LogEvent:
    source: str
    message: str


@dataclass(slots=True)
class ResultEvent:
    success: bool
    exit_code: int
    error: str


@dataclass(slots=True)
class TarArchiveStart:
    name: str


@dataclass(slots=True)
class TarDirectory:
    path: str
    mode: int
    uid: int
    gid: int
    uname: str
    gname: str
    mtime_unix: int


@dataclass(slots=True)
class TarSymlink:
    path: str
    target: str
    mode: int
    uid: int
    gid: int
    uname: str
    gname: str
    mtime_unix: int


@dataclass(slots=True)
class TarFilePart:
    path: str
    mode: int
    uid: int
    gid: int
    uname: str
    gname: str
    mtime_unix: int
    size: int
    offset: int
    data: bytes
    eof: bool


@dataclass(slots=True)
class DoneEvent:
    pass


ExecuteEvent = Union[
    AcceptedEvent,
    LogEvent,
    ResultEvent,
    TarArchiveStart,
    TarDirectory,
    TarSymlink,
    TarFilePart,
    DoneEvent,
]

ProxyStatusEvent = Union[
    ProxyStatusSummary,
    ProxyLayerstoreInstance,
    ProxyAfsletInstance,
    ProxyStatusError,
]



from __future__ import annotations

import argparse
import asyncio
import os
import pathlib
import stat
from typing import AsyncIterator, List

from afs_sdk import (
    AfsClient,
    DoneEvent,
    ExecuteInput,
    ExtraDirectory,
    ExtraFile,
    ExtraSymlink,
    LogEvent,
    ResultEvent,
    TarDirectory,
    TarFilePart,
    TarSymlink,
)


async def file_chunks(path: pathlib.Path, chunk_size: int) -> AsyncIterator[bytes]:
    with path.open("rb") as f:
        while True:
            chunk = f.read(chunk_size)
            if not chunk:
                break
            yield chunk


def build_entries(root_dir: str, chunk_size: int):
    if not root_dir:
        return []
    root = pathlib.Path(root_dir).resolve()
    out = []
    for p in sorted(root.rglob("*")):
        rel = p.relative_to(root).as_posix()
        st = p.lstat()
        mode = stat.S_IMODE(st.st_mode)
        if p.is_dir():
            out.append(ExtraDirectory(path=rel, mode=mode, uid=st.st_uid, gid=st.st_gid, mtime_unix=int(st.st_mtime)))
        elif p.is_symlink():
            out.append(
                ExtraSymlink(
                    path=rel,
                    target=os.readlink(p),
                    mode=mode,
                    uid=st.st_uid,
                    gid=st.st_gid,
                    mtime_unix=int(st.st_mtime),
                )
            )
        else:
            out.append(
                ExtraFile(
                    path=rel,
                    content=file_chunks(p, chunk_size),
                    mode=mode,
                    uid=st.st_uid,
                    gid=st.st_gid,
                    mtime_unix=int(st.st_mtime),
                )
            )
    return out


async def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("--addr", default="127.0.0.1:62051", help="afs_proxy/afslet address")
    parser.add_argument("--dir", default="", help="optional extra-dir to upload")
    parser.add_argument("--chunk-size", type=int, default=256 * 1024)
    parser.add_argument("--image", required=True)
    parser.add_argument("--tag", default="")
    parser.add_argument("--env", action="append", default=[], help="process environment override KEY=VALUE (repeatable)")
    parser.add_argument("--fuse-max-read-ahead-bytes", type=int, default=0, help="request-level FUSE read-ahead override in bytes")
    parser.add_argument("--cpu", type=int, default=1)
    parser.add_argument("--memory-mb", type=int, default=256)
    parser.add_argument("--timeout-ms", type=int, default=1000)
    parser.add_argument("command", nargs=argparse.REMAINDER)
    args = parser.parse_args()

    if args.command and args.command[0] == "--":
        args.command = args.command[1:]

    host, port_str = args.addr.rsplit(":", 1)
    port = int(port_str)

    req = ExecuteInput(
        image=args.image,
        tag=args.tag,
        command=args.command,
        env=args.env,
        fuse_max_read_ahead_bytes=args.fuse_max_read_ahead_bytes,
        cpu_cores=args.cpu,
        memory_mb=args.memory_mb,
        timeout_ms=args.timeout_ms,
        extra_entries=build_entries(args.dir, args.chunk_size),
        upload_chunk_size=args.chunk_size,
    )

    async with AfsClient(host, port) as client:
        async for ev in client.execute(req):
            if isinstance(ev, LogEvent):
                print(f"[log:{ev.source}] {ev.message}")
            elif isinstance(ev, ResultEvent):
                print(f"[result] success={ev.success} exit={ev.exit_code} err={ev.error!r}")
            elif isinstance(ev, TarDirectory):
                print(f"[tar:dir] {ev.path}")
            elif isinstance(ev, TarSymlink):
                print(f"[tar:symlink] {ev.path} -> {ev.target}")
            elif isinstance(ev, TarFilePart):
                print(f"[tar:file] {ev.path} offset={ev.offset} bytes={len(ev.data)} eof={ev.eof}")
            elif isinstance(ev, DoneEvent):
                print("[done]")


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations

import unittest

from afs_sdk import AfsClient, ExecuteInput


class ExecuteInputEnvTests(unittest.IsolatedAsyncioTestCase):
    async def test_iter_raw_requests_accepts_mapping_env(self) -> None:
        client = AfsClient("127.0.0.1", 1)
        try:
            reqs = client._iter_raw_requests(
                ExecuteInput(
                    image="alpine",
                    command=["/bin/sh", "-lc", "env"],
                    env={"FOO": "bar", "PATH": "/custom/bin"},
                )
            )

            first = await anext(reqs)

            self.assertEqual(first.start.image, "alpine")
            self.assertEqual(list(first.start.command), ["/bin/sh", "-lc", "env"])
            self.assertEqual(list(first.start.env), ["FOO=bar", "PATH=/custom/bin"])
        finally:
            await client.close()

    async def test_iter_raw_requests_rejects_non_string_env_entries(self) -> None:
        client = AfsClient("127.0.0.1", 1)
        try:
            reqs = client._iter_raw_requests(
                ExecuteInput(
                    image="alpine",
                    command=["/bin/true"],
                    env=["FOO=bar", 123],  # type: ignore[list-item]
                )
            )

            with self.assertRaises(TypeError):
                await anext(reqs)
        finally:
            await client.close()

    async def test_iter_raw_requests_includes_fuse_read_ahead_override(self) -> None:
        client = AfsClient("127.0.0.1", 1)
        try:
            reqs = client._iter_raw_requests(
                ExecuteInput(
                    image="alpine",
                    command=["/bin/true"],
                    fuse_max_read_ahead_bytes=16 << 20,
                )
            )

            first = await anext(reqs)

            self.assertEqual(first.start.fuse_max_read_ahead_bytes, 16 << 20)
        finally:
            await client.close()


if __name__ == "__main__":
    unittest.main()



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: api/afslet/v1/afslet.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

from . import afslet_pb2


class AfsletBase(abc.ABC):

    @abc.abstractmethod
    async def Execute(self, stream: 'grpclib.server.Stream[afslet_pb2.ExecuteRequest, afslet_pb2.ExecuteResponse]') -> None:
        pass

    @abc.abstractmethod
    async def GetRuntimeStatus(self, stream: 'grpclib.server.Stream[afslet_pb2.GetRuntimeStatusRequest, afslet_pb2.GetRuntimeStatusResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/afsletpb.Afslet/Execute': grpclib.const.Handler(
                self.Execute,
                grpclib.const.Cardinality.STREAM_STREAM,
                afslet_pb2.ExecuteRequest,
                afslet_pb2.ExecuteResponse,
            ),
            '/afsletpb.Afslet/GetRuntimeStatus': grpclib.const.Handler(
                self.GetRuntimeStatus,
                grpclib.const.Cardinality.UNARY_UNARY,
                afslet_pb2.GetRuntimeStatusRequest,
                afslet_pb2.GetRuntimeStatusResponse,
            ),
        }


class AfsletStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.Execute = grpclib.client.StreamStreamMethod(
            channel,
            '/afsletpb.Afslet/Execute',
            afslet_pb2.ExecuteRequest,
            afslet_pb2.ExecuteResponse,
        )
        self.GetRuntimeStatus = grpclib.client.UnaryUnaryMethod(
            channel,
            '/afsletpb.Afslet/GetRuntimeStatus',
            afslet_pb2.GetRuntimeStatusRequest,
            afslet_pb2.GetRuntimeStatusResponse,
        )



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: api/afsproxy/v1/afsproxy.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

from . import afsproxy_pb2


class AfsProxyBase(abc.ABC):

    @abc.abstractmethod
    async def Status(self, stream: 'grpclib.server.Stream[afsproxy_pb2.StatusRequest, afsproxy_pb2.StatusResponse]') -> None:
        pass

    @abc.abstractmethod
    async def ReconcileImageReplica(self, stream: 'grpclib.server.Stream[afsproxy_pb2.ReconcileImageReplicaRequest, afsproxy_pb2.ReconcileImageReplicaResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/afsproxy.v1.AfsProxy/Status': grpclib.const.Handler(
                self.Status,
                grpclib.const.Cardinality.UNARY_STREAM,
                afsproxy_pb2.StatusRequest,
                afsproxy_pb2.StatusResponse,
            ),
            '/afsproxy.v1.AfsProxy/ReconcileImageReplica': grpclib.const.Handler(
                self.ReconcileImageReplica,
                grpclib.const.Cardinality.UNARY_UNARY,
                afsproxy_pb2.ReconcileImageReplicaRequest,
                afsproxy_pb2.ReconcileImageReplicaResponse,
            ),
        }


class AfsProxyStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.Status = grpclib.client.UnaryStreamMethod(
            channel,
            '/afsproxy.v1.AfsProxy/Status',
            afsproxy_pb2.StatusRequest,
            afsproxy_pb2.StatusResponse,
        )
        self.ReconcileImageReplica = grpclib.client.UnaryUnaryMethod(
            channel,
            '/afsproxy.v1.AfsProxy/ReconcileImageReplica',
            afsproxy_pb2.ReconcileImageReplicaRequest,
            afsproxy_pb2.ReconcileImageReplicaResponse,
        )
